'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ms = _interopDefault(require('ms'));
var redis = require('redis');

var babelHelpers = {};

babelHelpers.classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

babelHelpers.createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

babelHelpers;

var RedisStore = function () {
	function RedisStore() {
		var _this = this,
		    _arguments = arguments;

		var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
		babelHelpers.classCallCheck(this, RedisStore);

		this.prefix = options.prefix == null ? 'jwt:' : options.prefix;
		delete options.prefix;

		if (options.url) {
			options.socket = options.url;
		}

		// convert to redis connect params
		if (options.client) {
			this.client = options.client;
		} else if (options.socket) {
			this.client = redis.createClient(options.socket, options);
		} else {
			this.client = redis.createClient(options);
		}

		this.ttl = options.ttl;
		var p = [];

		if (options.pass) p.push(new Promise(function (resolve, reject) {
			_this.client.auth(options.pass, function (e) {
				if (e) reject(e);else resolve();
			});
		}));

		if ('db' in options) {
			if (typeof options.db !== 'number') {
				console.error('Warning: redis store expects a number for the "db" option');
			}

			this.client.select(options.db);
			this.client.on('connect', function () {
				_this.client.select(options.db);
			});
		}

		p.push(new Promise(function (resolve, reject) {
			var onError = undefined,
			    onConnect = undefined;
			var done = function done(fn) {
				return function () {
					_this.client.removeListener('error', onError);
					_this.client.removeListener('connect', onConnect);
					fn.apply(null, _arguments);
				};
			};

			_this.client.on('error', onError = done(reject));
			_this.client.on('connect', onConnect = done(resolve));
		}));

		this._ready = Promise.all(p);
	}

	babelHelpers.createClass(RedisStore, [{
		key: "load",
		value: function load() {
			return this._ready;
		}
	}, {
		key: "_getKey",
		value: function _getKey(sid) {
			return this.prefix + sid;
		}
	}, {
		key: "add",
		value: function add(sid) {
			var _this2 = this;

			return new Promise(function (resolve, reject) {
				var key = _this2._getKey(sid);
				var value = Date.now();
				var done = function done(err) {
					if (err) reject(err);else resolve();
				};

				if (_this2.ttl) {
					_this2.client.setex(key, ms(String(_this2.ttl)), value, done);
				} else {
					_this2.client.set(key, value, done);
				}
			});
		}
	}, {
		key: "exists",
		value: function exists(sid) {
			var _this3 = this;

			return new Promise(function (resolve, reject) {
				_this3.client.exists(_this3._getKey(sid), function (err, res) {
					if (err) reject(err);else resolve(res ? true : false);
				});
			});
		}
	}, {
		key: "remove",
		value: function remove(sid) {
			var _this4 = this;

			return new Promise(function (resolve, reject) {
				_this4.client.del(_this4._getKey(sid), function (err, res) {
					if (err) reject(err);else resolve(res ? true : false);
				});
			});
		}
	}]);
	return RedisStore;
}();

function createRedisStore(options) {
	var store = new RedisStore(options);
	return store.load().then(function () {
		return store;
	});
}

module.exports = createRedisStore;